const { v4: uuidv4 } = require('uuid');

const write = require('../../../util/forum/write');
const addPostWithRetry = require('../../../util/add-post-with-retry');
const callWithRetry = require('../../../util/call-with-retry');
const { getBatchItems, clearBatchItems } = require('./batch-items');
const computeBatchPost = require('./compute-batch-post');
const { wallet, rollup } = require('../../../util/contracts');
const { sendMatrixEvent } = require('../../../matrix-bot');
const { stakeRollupAvailability } = require('../utils');
const fetchBatchItemsInfo = require('./fetch-batch-items-info');

const submitRollup = async () => {
  const availableBatchItems = getBatchItems();
  const batchItems = [];
  const batchItemsInfo = await fetchBatchItemsInfo();
  console.log('available batch items', availableBatchItems);
  for (let i = 0; i < batchItemsInfo.length; i += 1) {
    const { postId } = batchItemsInfo[i];
    if (availableBatchItems.includes(postId)) {
      console.log(`post ${postId} is available`);
      batchItems.push(postId);
    } else {
      // Batch items have to be submitted in the correct order, with no gaps
      console.log(`post ${postId} is not available`);
      break;
    }
  }
  if (!batchItems.length) {
    return { batchItems: [] };
  }
  const { authors, references } = await computeBatchPost(batchItems);
  if (!authors.length) {
    return { batchItems: [] };
  }
  const content = `Batch of ${batchItems.length} items`;
  const embeddedData = {
    batchItems,
    nonce: uuidv4().replaceAll('-', ''),
  };
  const sender = await wallet.getAddress();
  const contentToVerify = `${content}\n\n${JSON.stringify(embeddedData, null, 2)}`;
  const signature = await wallet.signMessage(contentToVerify);
  // Write to the forum database
  const { hash: batchPostId } = await write({
    sender, authors, references, content, embeddedData, signature,
  });
  // Add rollup post on-chain
  console.log('adding batch post on-chain', { authors, batchPostId, references });
  await addPostWithRetry(authors, batchPostId, references);
  // Stake our availability to be the next rollup worker
  console.log('staking availability to be the next rollup worker');
  await stakeRollupAvailability();
  // Call Rollup.submitBatch
  console.log('Submitting batch', { batchPostId, batchItems, authors });
  const poolDuration = 60;
  await callWithRetry(() => rollup.submitBatch(batchPostId, batchItems, poolDuration));
  // Send matrix event
  await sendMatrixEvent('io.dgov.rollup.submit', {
    batchPostId, batchItems, authors, references,
  });
  // Clear the batch in preparation for next batch
  await clearBatchItems(batchItems);
  return {
    batchPostId,
    batchItems,
    authors,
  };
};

module.exports = submitRollup;
